@@ -19,12 +19,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 """
 
 import socket, subprocess, time, datetime, os
+import csv
 
 from twisted.spread import pb
 from twisted.cred import credentials
 from twisted.internet.defer import Deferred
+from twisted.python.failure import Failure
 from twisted.internet.address import UNIXAddress
-from PyQt4.QtCore import SIGNAL, SLOT, Qt, QTimer
+from PyQt4.QtCore import SIGNAL, SLOT, Qt, QTimer, QObject
 from PyQt4.QtGui import QDialog, QDialogButtonBox, QVBoxLayout, QGridLayout, \
     QLabel, QComboBox, QLineEdit, QPushButton, \
     QProgressBar, QRadioButton, QSpacerItem, QSizePolicy
@@ -33,38 +35,44 @@ from PyKDE4.kdecore import KUser
 from PyKDE4.kdeui import KDialogButtonBox
 from PyKDE4.kdeui import KMessageBox
 
-from util import m18n, m18nc, logWarning, logException, syslogMessage, socketName, english
-from util import SERVERMARK
+from util import m18n, m18nc, logWarning, logException, socketName, english, \
+    appdataDir, logInfo
+from util import SERVERMARK, isAlive
 from message import Message
 import common
-from common import InternalParameters
+from common import InternalParameters, PREF
 from game import Players
-from query import Query
+from query import Transaction, Query
 from board import Board
-from client import Client
+from client import Client, Client1
 from statesaver import StateSaver
+from meld import Meld
 
 from guiutil import ListComboBox
-from scoringengine import Ruleset
+from scoringengine import Ruleset, PredefinedRuleset
 
 class LoginDialog(QDialog):
     """login dialog for server"""
     def __init__(self):
+        """self.servers is a list of tuples containing server and last playername"""
         QDialog.__init__(self, None)
         self.setWindowTitle(m18n('Login') + ' - Kajongg')
         self.setupUi()
 
         localName = m18nc('kajongg name for local game server', Query.localServerName)
+        self.servers = Query('select url,lastname from server order by lasttime desc').records
+        servers = [m18nc('kajongg name for local game server', x[0]) for x in self.servers]
+        # the first server combobox item should be default: either the last used server
+        # or localName for autoPlay
+        if localName not in servers:
+            servers.append(localName)
         if InternalParameters.autoPlay:
-            self.cbServer.addItem(localName)
-        self.servers = Query('select url, lastname from server order by lasttime desc').records
-        for server in self.servers:
-            if server[0] == Query.localServerName:
-                self.cbServer.addItem(localName)
-            else:
-                self.cbServer.addItem(server[0])
-        if self.cbServer.findText(localName) < 0:
-            self.cbServer.addItem(localName)
+            servers.remove(localName)    # we want a unique list, it will be re-used for all following games
+            servers.insert(0, localName)   # in this process but they will not be autoPlay
+        self.cbServer.addItems(servers)
+        self.passwords = Query('select url, p.name, passwords.password from passwords, player p '
+            'where passwords.player=p.id').records
+        Players.load()
         self.connect(self.cbServer, SIGNAL('editTextChanged(QString)'), self.serverChanged)
         self.connect(self.cbUser, SIGNAL('editTextChanged(QString)'), self.userChanged)
         self.serverChanged()
@@ -110,17 +118,16 @@ class LoginDialog(QDialog):
 
     def accept(self):
         """user entered OK"""
-        if self.host == Query.localServerName:
-            # client and server use the same database, and we
-            # have no security concerns
-            Players.createIfUnknown(self.host, unicode(self.cbUser.currentText()))
+        if self.host == 'localhost':
+            # we have localhost if we play a Local Game: client and server are identical,
+            # we have no security concerns about  creating a new account
+            Players.createIfUnknown(unicode(self.cbUser.currentText()))
         QDialog.accept(self)
 
     def serverChanged(self, dummyText=None):
         """the user selected a different server"""
-        Players.load()
         self.cbUser.clear()
-        self.cbUser.addItems(list(x[1] for x in Players.allNames.values() if x[0] == self.host))
+        self.cbUser.addItems(list(x[1] for x in self.servers if x[0] == self.host))
         if not self.cbUser.count():
             self.cbUser.addItem(KUser(os.geteuid()).fullName())
         hostName = self.host
@@ -143,10 +150,12 @@ class LoginDialog(QDialog):
         if text == '':
             self.edPassword.clear()
             return
-        passw = Query("select password from player where host=? and name=?",
-            list([self.host, unicode(text)])).records
+        passw = None
+        for entry in self.passwords:
+            if entry[0] == self.host and entry[1] == unicode(text):
+                passw = entry[2]
         if passw:
-            self.edPassword.setText(passw[0][0])
+            self.edPassword.setText(passw)
         else:
             self.edPassword.clear()
 
@@ -183,7 +192,7 @@ class LoginDialog(QDialog):
         return property(**locals())
 
     @apply
-    def password(): # pylint: disable-msg=E0202
+    def password(): # pylint: disable=E0202
         """abstracts the password of the dialog"""
         def fget(self):
             return unicode(self.edPassword.text())
@@ -193,8 +202,8 @@ class LoginDialog(QDialog):
 
 class AddUserDialog(QDialog):
     """add a user account on a server: This dialog asks for the needed attributes"""
-    # pylint: disable-msg=R0902
-    # pylint: we need more than 10 instance attributes
+    # pylint: disable=R0902
+    # pylint we need more than 10 instance attributes
 
     def __init__(self):
         QDialog.__init__(self, None)
@@ -234,7 +243,7 @@ class AddUserDialog(QDialog):
         pol.setHorizontalPolicy(QSizePolicy.Expanding)
         self.edUser.setSizePolicy(pol)
 
-        self.servers = Query('select url, lastname from server order by lasttime desc').records
+        self.servers = Query('select url from server order by lasttime desc').records
         for server in self.servers:
             if server[0] != Query.localServerName:
                 self.cbServer.addItem(server[0])
@@ -292,7 +301,7 @@ class AddUserDialog(QDialog):
         return property(**locals())
 
     @apply
-    def username(): # pylint: disable-msg=E0202
+    def username(): # pylint: disable=E0202
         """abstracts the username of the dialog"""
         def fget(self):
             return unicode(self.edUser.text())
@@ -301,7 +310,7 @@ class AddUserDialog(QDialog):
         return property(**locals())
 
     @apply
-    def password(): # pylint: disable-msg=E0202
+    def password(): # pylint: disable=E0202
         """abstracts the password of the dialog"""
         def fget(self):
             return unicode(self.edPassword.text())
@@ -313,6 +322,7 @@ class SelectChow(QDialog):
     """asks which of the possible chows is wanted"""
     def __init__(self, chows):
         QDialog.__init__(self)
+        self.setWindowTitle('Kajongg')
         self.chows = chows
         self.selectedChow = None
         layout = QVBoxLayout(self)
@@ -345,6 +355,43 @@ class SelectChow(QDialog):
         else:
             QDialog.keyPressEvent(self, event)
 
+class SelectKong(QDialog):
+    """asks which of the possible chows is wanted"""
+    def __init__(self, kongs):
+        QDialog.__init__(self)
+        self.setWindowTitle('Kajongg')
+        self.kongs = kongs
+        self.selectedKong = None
+        layout = QVBoxLayout(self)
+        layout.addWidget(QLabel(m18n('Which kong do you want to declare?')))
+        self.buttons = []
+        for kong in kongs:
+            button = QRadioButton(Meld.tileNames[kong[0][0].lower()] + ' ' + Meld.valueNames[kong[0][1]], self)
+            self.buttons.append(button)
+            layout.addWidget(button)
+            self.connect(button, SIGNAL('toggled(bool)'), self.toggled)
+
+    def toggled(self, dummyChecked):
+        """a radiobutton has been toggled"""
+        button = self.sender()
+        if button.isChecked():
+            self.selectedKong = self.kongs[self.buttons.index(button)]
+            self.accept()
+
+    def closeEvent(self, event):
+        """allow close only if a chow has been selected"""
+        if self.selectedKong:
+            event.accept()
+        else:
+            event.ignore()
+
+    def keyPressEvent(self, event):
+        """catch and ignore the Escape key"""
+        if event.key() == Qt.Key_Escape:
+            event.ignore()
+        else:
+            QDialog.keyPressEvent(self, event)
+
 class DlgButton(QPushButton):
     """special button for ClientDialog"""
     def __init__(self, key, parent):
@@ -370,11 +417,10 @@ class ClientDialog(QDialog):
         self.setWindowTitle(m18n('Choose') + ' - Kajongg')
         self.setObjectName('ClientDialog')
         self.client = client
-        self.relativePos = None
         self.layout = QGridLayout(self)
         self.progressBar = QProgressBar()
         self.timer = QTimer()
-        if not InternalParameters.autoPlay:
+        if not client.game.autoPlay:
             self.connect(self.timer, SIGNAL('timeout()'), self.timeout)
         self.deferred = None
         self.buttons = []
@@ -403,25 +449,26 @@ class ClientDialog(QDialog):
         self.connect(btn, SIGNAL('clicked(bool)'), self.selectedAnswer)
         self.buttons.append(btn)
 
-    def ask(self, move, answers, deferred):
+    def askHuman(self, move, answers, deferred):
         """make buttons specified by answers visible. The first answer is default.
         The default button only appears with blue border when this dialog has
         focus but we always want it to be recognizable. Hence setBackgroundRole."""
         self.move = move
         self.deferred = deferred
         for answer in answers:
-            self.__declareButton(answer)
+            if not PREF.showOnlyPossibleActions or self.client.maySay(self.move, answer):
+                self.__declareButton(answer)
         self.show()
         self.buttons[0].setFocus()
         myTurn = self.client.game.activePlayer == self.client.game.myself
-        if InternalParameters.autoPlay:
+        if self.client.game.autoPlay:
             self.selectButton()
             return
 
         self.progressBar.setVisible(not myTurn)
         if myTurn:
             hBoard = self.client.game.myself.handBoard
-            hBoard.showFocusRect(hBoard.focusTile)
+            hBoard.hasFocus = True
         else:
             msecs = 50
             self.progressBar.setMinimum(0)
@@ -441,7 +488,7 @@ class ClientDialog(QDialog):
             height = (len(self.buttons) + 1) * btnHeight * 1.2
             width = (cwi.width() - cwi.height() ) / 2
             geometry.setX(cwi.width() - width)
-            geometry.setY(cwi.height()/2  - height/2)
+            geometry.setY(min(cwi.height()/3,  cwi.height() - height))
         else:
             handBoard = self.client.game.myself.handBoard
             if not handBoard:
@@ -453,8 +500,6 @@ class ClientDialog(QDialog):
             height = btnHeight
             geometry.setY(cwi.height()  - height)
             geometry.setX(hbLeftTop.x())
-        spacer = QSpacerItem(20, 20, QSizePolicy.Expanding, QSizePolicy.Expanding)
-        self.layout.addItem(spacer, 0, 0)
         for idx, btn in enumerate(self.buttons + [self.progressBar]):
             self.layout.addWidget(btn, idx+1 if vertical else 0, idx+1 if not vertical else 0)
         idx = len(self.buttons) + 2
@@ -472,19 +517,24 @@ class ClientDialog(QDialog):
     def timeout(self):
         """the progressboard wants an update"""
         pBar = self.progressBar
-        pBar.setValue(pBar.value()+1)
-        pBar.setVisible(True)
-        if pBar.value() == pBar.maximum():
-            # timeout: we always return the original default answer, not the one with focus
-            self.selectButton()
-            pBar.setVisible(False)
+        if isAlive(pBar):
+            pBar.setValue(pBar.value()+1)
+            pBar.setVisible(True)
+            if pBar.value() == pBar.maximum():
+                # timeout: we always return the original default answer, not the one with focus
+                self.selectButton()
+                pBar.setVisible(False)
 
     def selectButton(self, button=None):
-        """select default answer"""
+        """select default answer. button may also be of type Message."""
         if self.isVisible():
             if button is None:
                 button = self.buttons[0]
-            answer = Message.defined[str(button.objectName())]
+            if isinstance(button, Message):
+                assert any(x.objectName() == button.name for x in self.buttons)
+                answer = button
+            else:
+                answer = Message.defined[str(button.objectName())]
             if not self.client.maySay(self.move, answer):
                 message = m18n('You cannot say %1', answer.i18nName)
                 KMessageBox.sorry(None, message)
@@ -527,13 +577,15 @@ class ReadyHandQuestion(QDialog):
             QDialog.keyPressEvent(self, event)
 
 
-class HumanClient(Client):
+class HumanClient(Client1):
     """a human client"""
+    # pylint: disable=R0904
+    # disable warning about too many public methods
     serverProcess = None
     socketServerProcess = None
 
     def __init__(self, tableList, callback):
-        Client.__init__(self)
+        Client1.__init__(self)
         self.root = None
         self.tableList = tableList
         self.connector = None
@@ -550,14 +602,31 @@ class HumanClient(Client):
             if not self.serverListening():
                 # give the server up to 5 seconds time to start
                 HumanClient.startLocalServer(self.useSocket)
-                for loop in range(5):
+                for loop in range(50):
                     if self.serverListening():
                         break
-                    time.sleep(1)
+                    time.sleep(0.1)
         self.username = self.loginDialog.username
-        self.ruleset = self.loginDialog.cbRuleset.current
+        self.ruleset = self.__defineRuleset()
         self.login(callback)
 
+    def __defineRuleset(self):
+        """find out what ruleset to use"""
+        if InternalParameters.autoPlayRuleset:
+            for ruleset in PredefinedRuleset.rulesets():
+                if ruleset.name == InternalParameters.autoPlayRuleset:
+                    return ruleset
+            try:
+                return Ruleset(InternalParameters.autoPlayRuleset)
+            except Exception as exception:
+                InternalParameters.autoPlay = False
+                InternalParameters.autoPlayRuleset = False
+                raise exception
+        elif InternalParameters.autoPlay:
+            return Ruleset.selectableRulesets()[0]
+        else:
+            return self.loginDialog.cbRuleset.current
+
     def isRobotClient(self):
         """avoid using isinstance, it would import too much for kajonggserver"""
         return False
@@ -584,7 +653,7 @@ class HumanClient(Client):
                 sock.connect(socketName())
             except socket.error:
                 if os.path.exists(socketName()):
-                    syslogMessage(m18n('removed stale socket <filename>%1</filename>', socketName()))
+                    logInfo(m18n('removed stale socket <filename>%1</filename>', socketName()))
                     os.remove(socketName())
                 return False
             else:
@@ -609,10 +678,13 @@ class HumanClient(Client):
             args = [cmd]
             if InternalParameters.showTraffic:
                 args.append('--showtraffic')
+            if InternalParameters.showSql:
+                args.append('--showsql')
             if useSocket:
-                args.append('--socket=%s' % socketName())
+                args.append('--socket')
+                args.append('--db=%slocal.db' % appdataDir())
             process = subprocess.Popen(args)
-            syslogMessage(m18n('started the local kajongg server: pid=<numid>%1</numid> %2',
+            logInfo(m18n('started the local kajongg server: pid=<numid>%1</numid> %2',
                 process.pid, ' '.join(args)))
             if useSocket:
                 HumanClient.socketServerProcess = process
@@ -626,7 +698,7 @@ class HumanClient(Client):
         """stop the local servers we started"""
         for process in [HumanClient.serverProcess, HumanClient.socketServerProcess]:
             if process:
-                syslogMessage(m18n('stopped the local kajongg server: pid=<numid>%1</numid>',
+                logInfo(m18n('stopped the local kajongg server: pid=<numid>%1</numid>',
                     process.pid))
                 process.terminate()
         HumanClient.serverProcess = None
@@ -636,13 +708,14 @@ class HumanClient(Client):
         """if we go away and we started a local server, stop it again"""
         HumanClient.stopLocalServers()
 
-    def remote_tablesChanged(self, tableid, tables):
+    def remote_tablesChanged(self, tables):
         """update table list"""
-        Client.remote_tablesChanged(self, tableid, tables)
+        Client.remote_tablesChanged(self, tables)
         self.tableList.load(self.tables)
 
-    def readyForGameStart(self, tableid, seed, playerNames, shouldSave=True):
+    def readyForGameStart(self, tableid, gameid, seed, playerNames, shouldSave=True):
         """playerNames are in wind order ESWN"""
+        self.tableList.hideForever = True
         if sum(not x.startswith('ROBOT') for x in playerNames.split('//')) == 1:
             # we play against 3 robots and we already told the server to start: no need to ask again
             wantStart = True
@@ -651,37 +724,41 @@ class HumanClient(Client):
                 "If you answer with NO, you will be removed from the table.")
             wantStart = KMessageBox.questionYesNo (None, msg) == KMessageBox.Yes
         if wantStart:
-            Client.readyForGameStart(self, tableid, seed, playerNames, shouldSave=shouldSave)
+            Client.readyForGameStart(self, tableid, gameid, seed, playerNames, shouldSave=shouldSave)
         else:
             self.answers.append(Message.NO)
 
-    def readyForHandStart(self, playerNames, rotate):
+    def readyForHandStart(self, playerNames, rotateWinds):
         """playerNames are in wind order ESWN"""
+        if InternalParameters.field:
+            # update the balances in the status bar:
+            InternalParameters.field.refresh()
         if self.game.handctr:
-            if InternalParameters.autoPlay:
-                self.clientReadyForHandStart(None, playerNames, rotate)
+            if self.game.autoPlay:
+                self.clientReadyForHandStart(None, playerNames, rotateWinds)
                 return
             deferred = Deferred()
-            deferred.addCallback(self.clientReadyForHandStart, playerNames, rotate)
+            deferred.addCallback(self.clientReadyForHandStart, playerNames, rotateWinds)
             self.readyHandQuestion = ReadyHandQuestion(deferred, InternalParameters.field)
             self.readyHandQuestion.show()
             self.answers.append(deferred)
 
-    def clientReadyForHandStart(self, dummy, playerNames, rotate):
+    def clientReadyForHandStart(self, dummy, playerNames, rotateWinds):
         """callback, called after the client player said yes, I am ready"""
-        Client.readyForHandStart(self, playerNames, rotate)
+        Client.readyForHandStart(self, playerNames, rotateWinds)
 
     def ask(self, move, answers, callback=None):
         """server sends move. We ask the user. answers is a list with possible answers,
         the default answer being the first in the list."""
+        if not InternalParameters.field:
+            return Client.ask(self, move, answers, callback)
         deferred = Deferred()
         if callback:
             deferred.addCallback(callback)
         deferred.addCallback(self.answered, move, answers)
         deferred.addErrback(self.answerError, move, answers)
-        handBoard = self.game.myself.handBoard
         iAmActive = self.game.myself == self.game.activePlayer
-        handBoard.setEnabled(iAmActive)
+        self.game.myself.handBoard.setEnabled(iAmActive)
         field = InternalParameters.field
         if not field.clientDialog or not field.clientDialog.isVisible():
             # always build a new dialog because if we change its layout before
@@ -689,12 +766,12 @@ class HumanClient(Client):
             # case the next dialog will appear at a lower position than it should
             field.clientDialog = ClientDialog(self, field.centralWidget())
         assert field.clientDialog.client is self
-        field.clientDialog.ask(move, answers, deferred)
+        field.clientDialog.askHuman(move, answers, deferred)
         self.answers.append(deferred)
 
     def selectChow(self, chows):
         """which possible chow do we want to expose?"""
-        if InternalParameters.autoPlay:
+        if self.game.autoPlay:
             return Client.selectChow(self, chows)
         if len(chows) == 1:
             return chows[0]
@@ -702,9 +779,19 @@ class HumanClient(Client):
         assert selDlg.exec_()
         return selDlg.selectedChow
 
+    def selectKong(self, kongs):
+        """which possible kong do we want to declare?"""
+        if self.game.autoPlay:
+            return Client.selectKong(self, kongs)
+        if len(kongs) == 1:
+            return kongs[0]
+        selDlg = SelectKong(kongs)
+        assert selDlg.exec_()
+        return selDlg.selectedKong
+
     def answered(self, answer, move, answers):
         """the user answered our question concerning move"""
-        if InternalParameters.autoPlay:
+        if self.game.autoPlay:
             self.game.hidePopups()
             return Client.ask(self, move, answers)
         myself = self.game.myself
@@ -720,7 +807,7 @@ class HumanClient(Client):
 
     def answerError(self, answer, move, answers):
         """an error happened while determining the answer to server"""
-        logException('%s %s %s' % (answer,  move,  answers))
+        logException('%s %s %s %s' % (self.game.myself.name, answer,  move,  answers))
 
     def remote_abort(self, tableid, message, *args):
         """the server aborted this game"""
@@ -730,19 +817,73 @@ class HumanClient(Client):
             logWarning(m18n(message, *args))
             if self.game:
                 self.game.close()
-            if InternalParameters.autoPlay:
-                if InternalParameters.field:
-                    InternalParameters.field.quit()
+                if self.game.autoPlay:
+                    if InternalParameters.field:
+                        InternalParameters.field.game = None
+                        InternalParameters.field.quit()
 
     def remote_gameOver(self, tableid, message, *args):
-        """the server aborted this game"""
+        """the game is over"""
         if self.table and self.table.tableid == tableid:
-            logWarning(m18n(message, *args))
+            if not self.game.autoPlay:
+                logInfo(m18n(message, *args), showDialog=True)
             if self.game:
                 self.game.rotateWinds()
+                if self.game.autoPlay and not InternalParameters.field:
+                    gameWinner = max(self.game.players, key=lambda x: x.balance)
+                    writer = csv.writer(open('kajongg.csv','a'), delimiter=';')
+                    row = [str(self.game.seed)]
+                    for player in sorted(self.game.players, key=lambda x: x.name):
+                        row.append(player.name)
+                        row.append(player.balance)
+                        row.append(player.wonCount)
+                        row.append(1 if player == gameWinner else 0)
+                    writer.writerow(row)
+                    del writer
                 self.game.close()
-            if InternalParameters.autoPlay:
-                InternalParameters.field.quit()
+                if self.game.autoPlay:
+                    self.abortGame(HumanClient.gameClosed)
+
+    def abortGame(self, callback=None):
+        """aborts current game"""
+        msg = m18n("Do you really want to abort this game?")
+        if self.game is None or self.game.autoPlay or \
+            self.game.finished() or \
+            KMessageBox.questionYesNo (None, msg) == KMessageBox.Yes:
+            if self.game:
+                self.game.close(callback)
+            if InternalParameters.field:
+                InternalParameters.field.game = None
+                InternalParameters.field.refresh()
+            return True
+        else:
+            return False
+
+    @staticmethod
+    def gameClosed(result=None):
+        """called if we want to quit, after the game has been closed"""
+        if isinstance(result, Failure):
+            logException(result)
+        InternalParameters.reactor.stop()
+        HumanClient.stopLocalServers()
+        # we may be in a Deferred callback generated in abortGame which would
+        # catch sys.exit as an exception
+        # and the qt4reactor does not quit the app when being stopped
+        QObject.connect(InternalParameters.app, SIGNAL('reactorStopped'), HumanClient.quit2)
+        QObject.emit(InternalParameters.app, SIGNAL('reactorStopped'))
+
+    @staticmethod
+    def quit2():
+        """2nd stage: twisted reactor is already stopped"""
+        StateSaver.saveAll()
+        InternalParameters.app.quit()
+    #       sys.exit(0)
+        # pylint: disable=W0212
+        os._exit(0) # TODO: should be sys.exit but that hangs since updating
+        # from karmic 32 bit to lucid 64 bit. os._exit does not clean up or flush buffers
+        # for reproduction, say "play" which opens the table list. Now close table list
+        # and try to quit.
+
 
     def remote_serverDisconnects(self):
         """the kajongg server ends our connection"""
@@ -752,28 +893,32 @@ class HumanClient(Client):
         """send a login command to server. That might be a normal login
         or adduser/deluser/change passwd encoded in the username"""
         factory = pb.PBClientFactory()
-        reactor = InternalParameters.field.reactor
+        reactor = InternalParameters.reactor
         if self.useSocket:
             self.connector = reactor.connectUNIX(socketName(), factory)
         else:
             self.connector = reactor.connectTCP(self.loginDialog.host, self.loginDialog.port, factory)
-        cred = credentials.UsernamePassword(username, self.loginDialog.password)
+        utf8Password = self.loginDialog.password.encode('utf-8')
+        utf8Username = username.encode('utf-8')
+        cred = credentials.UsernamePassword(utf8Username, utf8Password)
         return factory.login(cred, client=self)
 
     def adduser(self, host, name, passwd, callback, callbackParameter):
         """create  a user account"""
-        adduserDialog = AddUserDialog()
-        hostIdx = adduserDialog.cbServer.findText(host)
-        if hostIdx >= 0:
-            adduserDialog.cbServer.setCurrentIndex(hostIdx)
-        else:
-            adduserDialog.cbServer.insertItem(0, host)
-            adduserDialog.cbServer.setCurrentIndex(0)
-        adduserDialog.username = self.loginDialog.username
-        adduserDialog.password = self.loginDialog.password
-        if not adduserDialog.exec_():
-            raise Exception(m18n('Aborted creating a user account'))
-        name, passwd = adduserDialog.username, adduserDialog.password
+        assert host is not None
+        if self.loginDialog.host != Query.localServerName:
+            adduserDialog = AddUserDialog()
+            hostIdx = adduserDialog.cbServer.findText(host)
+            if hostIdx >= 0:
+                adduserDialog.cbServer.setCurrentIndex(hostIdx)
+            else:
+                adduserDialog.cbServer.insertItem(0, host)
+                adduserDialog.cbServer.setCurrentIndex(0)
+            adduserDialog.username = self.loginDialog.username
+            adduserDialog.password = self.loginDialog.password
+            if not adduserDialog.exec_():
+                raise Exception(m18n('Aborted creating a user account'))
+            name, passwd = adduserDialog.username, adduserDialog.password
         self.loginDialog.password = passwd
         adduserCmd =  SERVERMARK.join(['adduser', name, passwd])
         self.loginCommand(adduserCmd).addCallback(callback,
@@ -787,7 +932,8 @@ class HumanClient(Client):
         if 'Wrong username' in message:
             msg = m18nc('USER is not known on SERVER',
                 '%1 is not known on %2, do you want to open an account?', name, host)
-            if KMessageBox.questionYesNo (None, msg) == KMessageBox.Yes:
+            if self.loginDialog.host == Query.localServerName \
+            or KMessageBox.questionYesNo (None, msg) == KMessageBox.Yes:
                 self.adduser(host, name, passwd, self.adduserOK, callback)
                 return
         else:
@@ -797,7 +943,7 @@ class HumanClient(Client):
 
     def adduserOK(self, dummyFailure, callback):
         """adduser succeeded"""
-        Players.createIfUnknown(self.host, self.username)
+        Players.createIfUnknown(self.username)
         self.login(callback)
 
     def login(self, callback):
@@ -808,16 +954,24 @@ class HumanClient(Client):
     def loggedIn(self, perspective, callback):
         """we are online. Update table server and continue"""
         lasttime = datetime.datetime.now().replace(microsecond=0).isoformat()
-        qData = Query('select url from server where url=?',
-            list([self.host])).records
-        if not qData:
-            Query('insert into server(url,lastname,lasttime) values(?,?,?)',
-                list([self.host, self.username, lasttime]))
-        else:
-            Query('update server set lastname=?,lasttime=? where url=?',
-                list([self.username, lasttime, self.host]))
-            Query('update player set password=? where host=? and name=?',
-                list([self.loginDialog.password, self.host, self.username]))
+        host = english(self.host) # use unique name for Local Game
+        with Transaction():
+            qData = Query('select 1 from server where url=?',
+                list([host])).records
+            if not qData:
+                Query('insert into server(url,lastname,lasttime) values(?,?,?)',
+                    list([host, self.username, lasttime]))
+            else:
+                Query('update server set lastname=?,lasttime=? where url=?',
+                    list([self.username, lasttime, host]))
+                playerId = Players.allIds[self.username]
+                if Query('select 1 from passwords where url=? and player=?',
+                         list([host, playerId ])).records:
+                    Query('update passwords set password=? where url=? and player=?',
+                        list([self.loginDialog.password, host, playerId]))
+                else:
+                    Query('insert into passwords(url,player,password) values(?,?,?)',
+                        list([host,  playerId, self.loginDialog.password]))
         self.perspective = perspective
         if callback:
             callback()
@@ -860,3 +1014,18 @@ class HumanClient(Client):
                 self.perspective = None
                 logWarning(m18n('The connection to the server %1 broke, please try again later.',
                                   self.host))
+    def maySayChow(self, select=False):
+        """returns answer arguments for the server if calling chow is possible.
+        returns the meld to be completed"""
+        result = self.game.myself.possibleChows()
+        if result and select:
+            result = self.selectChow(result)
+        return result
+
+    def maySayKong(self, select=False):
+        """returns answer arguments for the server if calling or declaring kong is possible.
+        returns the meld to be completed or to be declared"""
+        result = self.game.myself.possibleKongs()
+        if result and select:
+            result = self.selectKong(result)
+        return result
