@@ -21,14 +21,14 @@ Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 Read the user manual for a description of the interface to this scoring engine
 """
 
-import re
+import re # the new regex is about 7% faster
 from hashlib import md5
 from timeit import Timer
 
 from PyQt4.QtCore import QString
 
-from util import m18n, m18nc, english, logException, debugMessage
-from common import InternalParameters, elements
+from util import m18n, m18nc, english, logException, logDebug
+from common import elements, Debug
 from query import Query
 from tile import chiNext
 from meld import Meld, NamedList, meldKey, Score, meldsContent, Pairs, \
@@ -52,8 +52,29 @@ class Ruleset(object):
         used rulesets and rules are stored in separate tables - this makes handling them easier.
         In table usedruleset the name is not unique.
     """
-    # pylint: disable-msg=R0902
-    # pylint: we need more than 10 instance attributes
+    # pylint: disable=R0902
+    # pylint we need more than 10 instance attributes
+
+    cache = dict()
+    hits = 0
+    misses = 0
+
+    @staticmethod
+    def clearCache():
+        """clears the cache with Rulesets"""
+        Ruleset.cache.clear()
+
+    @staticmethod
+    def cached(name, used=False):
+        """If a Ruleset instance is never changed, we can use a cache"""
+        cache = Ruleset.cache
+        cacheKey = str(name) + str(used)
+        if cacheKey in cache:
+            return cache[cacheKey]
+        result = Ruleset(name, used)
+        cache[cacheKey] = result
+        return result
+
 
     def __init__(self, name, used=False):
         self.name = name
@@ -87,13 +108,13 @@ class Ruleset(object):
         self.__minMJTotal = None
 
     @apply
-    def dirty(): # pylint: disable-msg=E0202
+    def dirty(): # pylint: disable=E0202
         """have we been modified since load or last save?"""
         def fget(self):
-            # pylint: disable-msg=W0212
+            # pylint: disable=W0212
             return self.__dirty
         def fset(self, dirty):
-            # pylint: disable-msg=W0212
+            # pylint: disable=W0212
             self.__dirty = dirty
             if dirty:
                 self.__computeHash()
@@ -103,7 +124,7 @@ class Ruleset(object):
     def hash():
         """a md5sum computed from the rules but not name and description"""
         def fget(self):
-            # pylint: disable-msg=W0212
+            # pylint: disable=W0212
             if not self.__hash:
                 self.__computeHash()
             return self.__hash
@@ -115,7 +136,7 @@ class Ruleset(object):
         the correct number is bigger in CC: 22 and not 20. But it is enough saveguard against
         entering impossible scores for manual games."""
         def fget(self):
-            # pylint: disable-msg=W0212
+            # pylint: disable=W0212
             if self.__minMJTotal is None:
                 self.__minMJTotal = self.minMJPoints + min(x.score.total(self.limit) for x in self.mjRules)
             return self.__minMJTotal
@@ -190,7 +211,7 @@ class Ruleset(object):
 
     def loadRule(self, record):
         """loads a rule into the correct ruleList"""
-        (rulesetIdx, name, listNr, position, definition, points, doubles, limits, # pylint: disable-msg=W0612
+        (rulesetIdx, name, listNr, position, definition, points, doubles, limits, # pylint: disable=W0612
             parameter) = record
         for ruleList in self.ruleLists:
             if ruleList.listId == listNr:
@@ -247,7 +268,7 @@ class Ruleset(object):
                 'Copy%1 of %2', copyStr, m18n(self.name))
             if not self.nameIsDuplicate(newName):
                 return newId, newName
-        logException(Exception(m18n('You already have the maximum number of copies, please rename some')))
+        logException('You already have the maximum number of copies, please rename some')
 
     def ruleNameIsDuplicate(self, name):
         """True if a rule with name already exists"""
@@ -302,7 +323,7 @@ class Ruleset(object):
                 ruleList = self.__ruleList(rule)
                 ruleList.insert(ruleList.index(rule) + 1, result)
                 return result
-        logException(Exception(m18n('You already have the maximum number of copies, please rename some')))
+        logException('You already have the maximum number of copies, please rename some')
 
     def __rulesetTable(self):
         """the table name for the ruleset"""
@@ -390,7 +411,7 @@ class Ruleset(object):
         return [Ruleset(x) for x in Ruleset.availableRulesetNames()]
 
     @staticmethod
-    def selectableRulesets(server):
+    def selectableRulesets(server=None):
         """returns all selectable rulesets for a new game.
         server is used to find the last ruleset used by us on that server, this
         ruleset will returned first in the list."""
@@ -398,20 +419,26 @@ class Ruleset(object):
         # if we have a selectable ruleset with the same name as the last used ruleset
         # put that ruleset in front of the list. We do not want to use the exact same last used
         # ruleset because we might have made some fixes to the ruleset meanwhile
-        if server is None:
-            server = ''
-        qData = Query("select ruleset from game where server=? order by starttime desc limit 1",
-            list([server])).records
-        if not qData:
-            # if client and server use the same data base, the server name is not stored as expected
-            qData = Query("select ruleset from game order by starttime desc limit 1").records
+        if server is None: # scoring game
+            # the exists clause is only needed for inconsistent data bases
+            qData = Query("select ruleset from game where seed is null "
+                " and exists(select id from usedruleset where game.ruleset=usedruleset.id)"
+                "order by starttime desc limit 1").records
+        else:
+            qData = Query('select lastruleset from server where lastruleset is not null and url=?',
+                list([server])).records
+            if not qData:
+                # we never played on that server
+                qData = Query('select lastruleset from server where lastruleset is not null '
+                    'order by lasttime desc limit 1').records
         if qData:
             qData = Query("select name from usedruleset where id=%d" % qData[0][0]).records
-            lastUsed = qData[0][0]
-            for idx, ruleset in enumerate(result):
-                if ruleset.name == lastUsed:
-                    del result[idx]
-                    return [ruleset ] + result
+            if qData:
+                lastUsed = qData[0][0]
+                for idx, ruleset in enumerate(result):
+                    if ruleset.name == lastUsed:
+                        del result[idx]
+                        return [ruleset ] + result
         return result
 
     def diff(self, other):
@@ -433,8 +460,8 @@ class Ruleset(object):
 class HandContent(object):
     """represent the hand to be evaluated"""
 
-    # pylint: disable-msg=R0902
-    # pylint: we need more than 10 instance attributes
+    # pylint: disable=R0902
+    # pylint we need more than 10 instance attributes
 
     cache = dict()
     hits = 0
@@ -443,7 +470,7 @@ class HandContent(object):
     @staticmethod
     def clearCache():
         """clears the cache with HandContents"""
-        #debugMessage('cache hits:%d misses:%d' % (HandContent.hits,  HandContent.misses))
+        #logDebug('cache hits:%d misses:%d' % (HandContent.hits,  HandContent.misses))
         HandContent.cache.clear()
         HandContent.hits = 0
         HandContent.misses = 0
@@ -466,7 +493,7 @@ class HandContent(object):
     def __init__(self, ruleset, string, computedRules=None, plusTile=None, robbedTile=None):
         """evaluate string using ruleset. rules are to be applied in any case."""
         # silence pylint. This method is time critical, so do not split it into smaller methods
-        # pylint: disable-msg=R0902,R0914,R0912,R0915
+        # pylint: disable=R0902,R0914,R0912,R0915
         self.ruleset = ruleset
         self.string = string
         self.plusTile = plusTile
@@ -540,7 +567,7 @@ class HandContent(object):
         if len(variants) > 1:
             if variants[1][0].total(self.ruleset.limit) > variants[0][0].total(self.ruleset.limit):
                 chosenVariant = variants[1]
-        score, rules, won = chosenVariant # pylint: disable-msg=W0612
+        score, rules, won = chosenVariant # pylint: disable=W0612
         exclusive = self.__exclusiveRules(rules)
         if exclusive:
             self.usedRules = exclusive
@@ -549,6 +576,17 @@ class HandContent(object):
             self.usedRules.extend(rules)
             self.score = score
 
+
+    def __sub__(self, tiles):
+        """returns a copy of self minus tiles"""
+        string = self.string
+        if not isinstance(tiles, list):
+            tiles = list([tiles])
+        for tile in tiles:
+            assert isinstance(tile, str) and len(tile) == 2,  'HandContent.__sub__:%s' % tiles
+            string = string.replace(tile, '', 1)
+        return HandContent.cached(self.ruleset, string, self.computedRules)
+
     def ruleMayApply(self, rule):
         """returns True if rule applies to either original or normalized"""
         return rule.appliesToHand(self, self.original) or rule.appliesToHand(self, self.normalized)
@@ -577,26 +615,25 @@ class HandContent(object):
         return tileCount - kongCount - 13
 
     def isCalling(self):
-        """the hand is calling if it only needs one tile for mah jongg"""
+        """the hand is calling if it only needs one tile for mah jongg.
+        Returns a tile needed for MJ or None"""
         if self.handLenOffset():
-            return False
+            return None
         # here we assume things about the possible structure of a
         # winner hand. Recheck this when supporting new exotic hands.
         if len(self.melds) > 7:
             # only possibility is 13 orphans
             if any(x in self.tiles.lower() for x in '2345678'):
                 # no minors allowed
-                return False
-            if sum(x in self.tiles.lower() for x in elements.majors) <12:
-                # not enough different majors
-                return False
-            return True
+                return None
+            tiles = sum((x.pairs for x in self.melds), [])
+            missing = elements.majors - set(x.lower() for x in tiles)
+            # if all 13 tiles are there, we need any one of them:
+            return (missing or elements.majors).pop()
         # no other legal winner hand allows singles that are not adjacent
         # to any other tile, so we only try tiles on the hand and for the
         # suit tiles also adjacent tiles
-        hiddenTiles = []
-        for meld in self.hiddenMelds:
-            hiddenTiles.extend(meld.pairs)
+        hiddenTiles = sum((x.pairs for x in self.hiddenMelds), [])
         checkTiles = set(hiddenTiles)
         for tile in hiddenTiles:
             if tile[0] in 'SBC':
@@ -607,7 +644,7 @@ class HandContent(object):
         for tile in checkTiles:
             hand = HandContent.cached(self.ruleset, self.string, plusTile=tile)
             if hand.maybeMahjongg():
-                return True
+                return tile
 
     def maybeMahjongg(self, checkScore=True):
         """check if this hand can be a regular mah jongg.
@@ -802,10 +839,8 @@ class HandContent(object):
                 rest[0] += self.plusTile
             else:
                 rest.append(self.plusTile)
-        if len(rest) > 1:
-            raise Exception('hand has more than 1 unsorted part: ', self.original)
         if rest:
-            rest = rest[0]
+            rest = ''.join(rest)
             rest = ''.join(sorted([rest[x:x+2] for x in range(0, len(rest), 2)]))
             self.melds |= self.split(rest)
         self.categorizeMelds()
@@ -853,7 +888,7 @@ class HandContent(object):
     def summary():
         """returns a summarizing string for this hand"""
         def fget(self):
-            # pylint: disable-msg=W0212
+            # pylint: disable=W0212
             if self.__summary is None:
                 handlenOffs = self.handLenOffset()
                 if handlenOffs < 0:
@@ -882,14 +917,18 @@ class Rule(object):
     The rule applies if at least one of the variants matches the hand.
     For parameter rules, only use name, definition,parameter. definition must start with int or str
     which is there for loading&saving, but internally is stripped off."""
-    # pylint: disable-msg=R0902
-    # pylint: we need more than 10 instance attributes
+    # pylint: disable=R0913,R0902
+    # pylint we need more than 10 instance attributes
 
-    def __init__(self, name, definition, points = 0, doubles = 0, limits = 0, parameter = None, debug=False):
+    functions = {}
+
+    def __init__(self, name, definition, points = 0, doubles = 0, limits = 0, parameter = None,
+            description=None, debug=False):
         self.actions = {}
         self.manualRegex = None
         self.variants = []
         self.name = name
+        self.description = description
         self.score = Score(points, doubles, limits)
         self._definition = None
         self.parName = ''
@@ -910,11 +949,11 @@ class Rule(object):
         self.definition = definition
 
     @apply
-    def definition(): # pylint: disable-msg=E0202
+    def definition(): # pylint: disable=E0202
         """the rule definition. See user manual about ruleset."""
-        # pylint: disable-msg=R0912
+        # pylint: disable=R0912
         def fget(self):
-            # pylint: disable-msg=W0212
+            # pylint: disable=W0212
             if isinstance(self._definition, list):
                 return '||'.join(self._definition)
             return self._definition
@@ -934,7 +973,9 @@ class Rule(object):
             for variant in variants:
                 if isinstance(variant, (str, unicode)):
                     variant = str(variant)
-                    if variant[0] == 'I':
+                    if variant[0] == 'F':
+                        self.variants.append(Rule.functions[variant[1:]])
+                    elif variant[0] == 'I':
                         self.variants.append(RegexIgnoringCase(self, variant[1:]))
                     elif variant[0] == 'A':
                         for action in variant[1:].split():
@@ -955,8 +996,8 @@ class Rule(object):
         payees = int(self.actions.get('payees', 1))
         if not 2 <= payers + payees <= 4:
             self.definition = prevDefinition
-            logException(Exception(m18nc('%1 can be a sentence', '%4 have impossible values %2/%3 in rule "%1"',
-                                  self.name, payers, payees, 'payers/payees')))
+            logException(m18nc('%1 can be a sentence', '%4 have impossible values %2/%3 in rule "%1"',
+                                  self.name, payers, payees, 'payers/payees'))
 
     def appliesToHand(self, hand, melds):
         """does the rule apply to this hand?"""
@@ -999,11 +1040,12 @@ class Rule(object):
     def copy(self):
         """returns a deep copy of self"""
         return Rule(self.name, self.definition, self.score.points, self.score.doubles,
-            self.score.limits, self.parameter)
+            self.score.limits, self.parameter, self.description)
 
-    def exclusive(self):
+    @staticmethod
+    def exclusive():
         """True if this rule can only apply to one player"""
-        return 'payforall' in self.actions
+        return False
 
     def hasNonValueAction(self):
         """Rule has a special action not changing the score directly"""
@@ -1019,7 +1061,7 @@ class Regex(object):
         try:
             self.compiled = re.compile(definition)
         except Exception, eValue:
-            logException(Exception('%s %s: %s' % (rule.name, definition, eValue)))
+            logException('%s %s: %s' % (rule.name, definition, eValue))
             raise
 
     def appliesToHand(self, hand, melds, debug=False):
@@ -1030,13 +1072,13 @@ class Regex(object):
         else:
             checkStr = meldStr + ' ' + hand.mjStr
         str2 = ' ,,, '.join((checkStr, checkStr))
-        if InternalParameters.profileRegex:
-            self.timeSum += Timer(stmt='x.search("%s")'%str2, setup="""import re
+        if Debug.profileRegex:
+            self.timeSum += Timer(stmt='x.search("%s")'%str2, setup="""import re as re
 x=re.compile(r"%s")"""%self.definition).timeit(50)
             self.count += 1
         match = self.compiled.search(str2)
-        if debug or InternalParameters.debugRegex:
-            debugMessage( '%s: %s against %s %s' % ('MATCH:' if match else 'NO MATCH:', \
+        if debug or Debug.regex:
+            logDebug( '%s: %s against %s %s' % ('MATCH:' if match else 'NO MATCH:', \
                 str2, self.rule.name, self.definition))
         return match
 
@@ -1047,11 +1089,78 @@ x=re.compile(r"%s")"""%self.definition).timeit(50)
         else:
             checkStr = meld.joined + ' ' + hand.mjStr
         match = self.compiled.match(checkStr)
-        if InternalParameters.debugRegex and match:
-            debugMessage('%s %s against %s %s' % ('MATCH:' if match else 'NO MATCH:',
+        if Debug.regex and match:
+            logDebug('%s %s against %s %s' % ('MATCH:' if match else 'NO MATCH:',
                 meld.joined + ' ' + hand.mjStr, self.rule.name, self.rule.definition))
         return match
 
+class Function(object):
+    """Parent for all Function classes. We need to implement
+    those methods as in Regex:
+    appliesToHand and appliesToMeld"""
+    def __init__(self):
+        self.timeSum = 0.0
+        self.count = 0
+        self.definition = ''
+
+    @staticmethod
+    def appliesToMeld(dummyHand, dummyMeld):
+        """we normally do not use this one"""
+        return False
+
+class FunctionLastOnlyPossible(Function):
+    """check if the last tile was the only one possible for winning"""
+
+    @staticmethod
+    def appliesToHand(hand, dummyMelds, dummyDebug=False):
+        """see class docstring"""
+        # pylint: disable=R0911
+        # pylint: disable=R0912
+        lastMeld = hand.mjStr.split(' L')
+        if len(lastMeld) < 2 or len(lastMeld[1]) == 0:
+            # no last tile specified
+            return False
+        lastMeld = lastMeld[1].split()[0]
+        lastTile = lastMeld[:2]
+        lastMeld = Meld(lastMeld[2:])
+        if len(lastMeld) == 0:
+            # no last meld specified: This can happen if we only want to
+            # know if saying Mah Jongg is possible
+            return False
+        if lastMeld.isSingle():
+            # a limit hand, this rule does not matter anyway
+            return False
+        if lastMeld.isPung():
+            return False # we had two pairs...
+        group, value = lastTile
+        group = group.lower()
+        if  group not in 'sbc':
+            return True
+        intValue = int(value)
+        if lastMeld.isChow():
+            if lastTile != lastMeld.pairs[1]:
+                # left or right tile of a chow:
+                if not ((value == '3' and lastMeld.pairs[0][1] == '1')
+                        or (value == '7' and lastMeld.pairs[0][1] == '9')):
+                    return False
+        else:
+            assert lastMeld.isPair()
+            for meld in hand.hiddenMelds:
+                # look at other hidden melds of same color:
+                if meld != lastMeld and meld.pairs[0][0].lower() == group:
+                    if meld.isChow():
+                        if intValue in [int(meld.pairs[0][1]) - 1,  int(meld.pairs[2][1]) + 1]:
+                            # pair and adjacent Chow
+                            return False
+                    elif meld.isPung():
+                        if abs(intValue - int(meld.pairs[0][1])) <= 2:
+                            # pair and nearby Pung
+                            return False
+                    elif meld.isSingle():
+                        # must be 13 orphans
+                        return False
+        return True
+
 class RegexIgnoringCase(Regex):
     """this Regex ignores case on the meld strings"""
     pass
@@ -1130,5 +1239,18 @@ def testScoring():
     pair1.toLower(3)
     assert pair1 ==  pair2
 
+def __scanSelf():
+    """for every Function class defined in this module,
+    generate an instance and add it to dict Rule.functions"""
+    if not Rule.functions:
+        for glob in globals().values():
+            if hasattr(glob, "__mro__"):
+                if glob.__mro__[-2] == Function and len(glob.__mro__) > 2:
+                    fun = glob()
+                    name = fun.__class__.__name__.replace('Function', '')
+                    Rule.functions[name] = fun
+
+__scanSelf()
+
 if __name__ == "__main__":
-    testScoring()
+    testScoring()
\ No newline at end of file
