@@ -83,7 +83,7 @@ BoardWidget::BoardWidget( QWidget* parent )
 }
 
 BoardWidget::~BoardWidget(){
-  if (Game) delete Game;
+  delete Game;
 }
 
 void BoardWidget::loadSettings(){
@@ -688,7 +688,7 @@ void BoardWidget::calculateNewGame( int gNumber)
         }
     }
 
-    drawBoard(true);
+    drawBoard(false);
     setStatusText( i18n("Error generating new game!") );
 }
 
@@ -779,10 +779,7 @@ void BoardWidget::removeTile( POSITION& Pos , bool doRepaint)
     Game->TileNum--;                    // Eine Figur weniger
     Game->setMoveListData(Game->TileNum,Pos); // Position ins Protokoll eintragen
 
-    TileSprite * thissprite =spriteMap.value(TileCoord(X,Y,E));
-    if (thissprite) delete thissprite;
-
-    spriteMap.remove(TileCoord(X,Y,E));
+    delete spriteMap.take(TileCoord(X,Y,E));
     // remove tile from game board
     Game->putTile( E, Y, X, 0 );
 }
@@ -893,11 +890,6 @@ void BoardWidget::transformPointToPosition(
     }
 
     TileCoord coord = spriteMap.key(clickedItem);
-    if (coord.isNull()) {
-	qDebug() << "null tile coordinates, clicked on background?";
-	return;
-    }
-
     E = coord.z();
     X = coord.x();
     Y = coord.y();
@@ -941,7 +933,7 @@ void BoardWidget::transformPointToPosition(
 // ---------------------------------------------------------
 bool BoardWidget::loadBoard( )
 {
-    if (Game) delete Game;
+    delete Game;
     Game = new GameData(theBoardLayout.board());
     return(true);
 }
