@@ -24,10 +24,12 @@ Q_OBJECT
     QLabel* m_time_label;
     Ui_Preferences m_pref_ui;
     MainArea* m_main;
+    QPair<QByteArray, QString> m_lastUsedDifficulty;
     
     void showCursor(bool show);
 public:
     MainWindow();
+    ~MainWindow();
 protected:
     void setupActions();
 protected slots:
