@@ -1,7 +1,7 @@
 /*
  * Copyright (C) 1995 Paul Olav Tvete <paul@troll.no>
  * Copyright (C) 2000-2009 Stephan Kulow <coolo@kde.org>
- * Copyright (C) 2009 Parker Coates <parker.coates@gmail.com>
+ * Copyright (C) 2009-2010 Parker Coates <parker.coates@kdemail.net>
  *
  * License of original code:
  * -------------------------------------------------------------------------
@@ -38,33 +38,23 @@
 #ifndef DEALER_H
 #define DEALER_H
 
-#include "card.h"
-#include "cardscene.h"
-class CardState;
 class DealerScene;
+class GameState;
 class MoveHint;
-#include "pile.h"
-class Solver;
+#include "patpile.h"
+#include "speeds.h"
 #include "view.h"
+class Solver;
 
-#include <KRandomSequence>
+#include "KCardDeck"
+#include "KCardScene"
 
-#include <QtCore/QSet>
-#include <QtGui/QAction>
-#include <QtGui/QGraphicsScene>
+class QAction;
+#include <QtCore/QMap>
 class QDomDocument;
 
 
-typedef QList<Pile*> PileList;
-typedef QList<CardState> CardStateList;
-
-struct State
-{
-    CardStateList cards;
-    QString gameData;
-};
-
-class DealerScene : public CardScene
+class DealerScene : public KCardScene
 {
     Q_OBJECT
 
@@ -74,44 +64,49 @@ public:
     DealerScene();
     ~DealerScene();
 
+    virtual void initialize() = 0;
+
     virtual void relayoutScene();
+    void updateWonItem();
 
-    virtual bool checkPrefering( int checkIndex, const Pile *c1, const CardList& c2) const;
 
-    // use this for autodrop times
-    int speedUpTime( int delay ) const;
+    virtual void addPile( KCardPile * pile );
+    virtual void removePile( KCardPile * pile );
+    QList<PatPile*> patPiles() const;
 
-    void createDump( QPaintDevice * );
+    virtual void moveCardsToPile( QList<KCard*> cards, KCardPile * pile, int duration );
 
-    void setAutoDropEnabled(bool a);
-    bool autoDropEnabled() const { return _autodrop; }
+    void setAutoDropEnabled( bool enabled );
+    bool autoDropEnabled() const;
 
-    void setGameNumber(int gmn);
     int gameNumber() const;
 
-    void setGameId(int id);
+    void setGameId( int id );
     int gameId() const;
 
-    void setActions(int actions);
+    void setActions( int actions );
     int actions() const;
 
     virtual QList<QAction*> configActions() const;
 
-    void setSolverEnabled(bool a);
-    void setSolver( Solver *s);
-    Solver *solver() const;
+    void startHint();
+    void stopHint();
+    bool isHintActive() const;
+ 
+    void startDrop();
+    void stopDrop();
+    bool isDropActive() const;
+
+    void startDemo();
+    void stopDemo();
+    bool isDemoActive() const;
+
+    void setSolverEnabled( bool enabled );
+    Solver * solver() const;
     void startSolver() const;
-    void unlockSolver() const;
-    void finishSolver() const;
 
-    void setNeededFutureMoves(int);
-    int neededFutureMoves() const;
-
-    bool demoActive() const;
-    virtual bool isGameLost() const;
-    virtual bool isGameWon() const;
     bool allowedToStartNewGame();
-    int getMoves() const;
+    int moveCount() const;
 
     QString save_it();
     void saveGame(QDomDocument &doc);
@@ -120,54 +115,56 @@ public:
     virtual int oldId() const;
     void recordGameStatistics();
 
-    virtual void restart() = 0;
+    QImage createDump() const;
 
 signals:
     void undoPossible(bool poss);
     void redoPossible(bool poss);
     void hintPossible(bool poss);
     void demoPossible(bool poss);
+    void dropPossible(bool poss);
     void newCardsPossible(bool poss);
 
-    void demoActive(bool en);
+    void demoActive(bool active);
+    void hintActive(bool active);
+    void dropActive(bool active);
     void updateMoves(int moves);
 
     void solverStateChanged(QString text);
 
+    void cardsPickedUp();
+    void cardsPutDown();
+
 public slots:
     void startNew(int gameNumber = -1);
-    void hint();
 
     void undo();
     void redo();
 
+    void stop();
+
+    void drawDealRowOrRedeal();
+
 protected:
-    virtual void mouseDoubleClickEvent ( QGraphicsSceneMouseEvent * mouseEvent );
-    virtual void mousePressEvent ( QGraphicsSceneMouseEvent * mouseEvent );
-    virtual void mouseReleaseEvent ( QGraphicsSceneMouseEvent * mouseEvent );
+    virtual bool allowedToAdd(const KCardPile * pile, const QList<KCard*> & cards) const;
+    virtual bool allowedToRemove(const KCardPile * pile, const KCard * card) const;
 
-    virtual bool cardDoubleClicked( Card * card );
+    virtual bool checkAdd( const PatPile * pile, const QList<KCard*> & oldCards, const QList<KCard*> & newCards ) const;
+    virtual bool checkRemove( const PatPile * pile, const QList<KCard*> & cards ) const;
+    virtual bool checkPrefering( const PatPile * pile, const QList<KCard*> & oldCards, const QList<KCard*> & newCards ) const;
 
-    virtual void onGameStateAlteredByUser();
+    virtual void mouseDoubleClickEvent( QGraphicsSceneMouseEvent * mouseEvent );
+    virtual void mousePressEvent( QGraphicsSceneMouseEvent * mouseEvent );
+    virtual void mouseReleaseEvent( QGraphicsSceneMouseEvent * mouseEvent );
 
-    State *getState();
-    void setState(State *);
-    void eraseRedo();
+    virtual void restart( const QList<KCard*> & cards ) = 0;
 
-    Pile *findTarget(Card *c);
+    void setSolver( Solver * solver );
 
-    QList<MoveHint*> hints() const;
-    virtual void getHints();
-    void getSolverHints();
-    void newHint(MoveHint *mh);
-    void clearHints();
-    // it's not const because it changes the random seed
-    virtual MoveHint *chooseHint();
+    virtual bool isGameLost() const;
+    virtual bool isGameWon() const;
 
-    virtual void stopDemo();
-
-    void won();
-    void updateWonItem();
+    virtual QList<MoveHint> getHints();
 
     // reimplement these to store and load game-specific information in the state structure
     virtual QString getGameState() { return QString(); }
@@ -177,41 +174,48 @@ protected:
     virtual QString getGameOptions() const { return QString(); }
     virtual void setGameOptions( const QString & ) {}
 
-    virtual void newDemoMove(Card *m);
-
-    void addCardForDeal( Pile * pile, Card * card, bool faceUp, QPointF startPos );
+    void addCardForDeal( KCardPile * pile, KCard * card, bool faceUp, QPointF startPos );
     void startDealAnimation();
 
-protected slots:
-    virtual void demo();
-    void waitForDemo(Card *);
-    void toggleDemo();
+    void setNeededFutureMoves( int moves );
+    int neededFutureMoves() const;
 
+    void setDeckContents( int copies = 1,
+                          const QList<KCardDeck::Suit> & suits = KCardDeck::standardSuits() );
+
+protected slots:
+    void takeState();
+    virtual void animationDone();
+    virtual bool newCards();
+    virtual bool drop();
+    virtual bool tryAutomaticMove( KCard * card );
+
+    void undoOrRedo( bool undo );
+    GameState * getState();
+    void setState( GameState * state );
+
+    QList<MoveHint> getSolverHints();
+
+private slots:
+    void stopAndRestartSolver();
     void slotSolverEnded();
-    void slotSolverFinished();
-    void startManualDrop();
+    void slotSolverFinished( int result );
+
+    void demo();
 
     void showWonMessage();
 
-    void takeState();
-    virtual Card *newCards();
-    virtual bool drop();
-
 private:
     void resetInternals();
 
-    bool _autodrop;
-    bool _usesolver;
-    int gamenumber;
+    MoveHint chooseHint();
+
+    void won();
+
+    int speedUpTime( int delay ) const;
 
     class DealerScenePrivate;
     DealerScenePrivate *d;
-
-    QMap<Card*,QPointF> m_initDealPositions;
-
-private slots:
-    void waitForAutoDrop();
-    void stopAndRestartSolver();
 };
 
 #endif
