@@ -43,53 +43,53 @@ class KlondikePile;
 class KSelectAction;
 
 
-class Klondike : public DealerScene {
-    friend class KlondikeSolver;
-
+class Klondike : public DealerScene
+{
     Q_OBJECT
 
 public:
-    explicit Klondike();
-
+    virtual void initialize();
     virtual void mapOldId(int id);
     virtual int oldId() const;
-    virtual void restart();
-    virtual bool drop();
     QList<QAction*> configActions() const;
 
-    void deal();
-
-    void setEasy( bool easy );
-
-public slots:
-    virtual Card *newCards();
-
 protected:
     virtual QString getGameState();
     virtual QString getGameOptions() const;
     virtual void setGameOptions(const QString &options);
+    virtual bool checkAdd(const PatPile * pile, const QList<KCard*> & oldCards, const QList<KCard*> & newCards) const;
+    virtual bool checkRemove(const PatPile * pile, const QList<KCard*> & cards) const;
+    virtual void restart( const QList<KCard*> & cards );
+
+protected slots:
+    virtual bool drop();
+    virtual bool newCards();
+
+private slots:
+    void gameTypeChanged();
 
 private:
+    void setEasy( bool easy );
+
     bool EasyRules;
     bool redealt;
 
     KSelectAction *options;
 
-    Pile* talon;
-    Pile* play[7];
-    Pile* target[4];
+    PatPile* talon;
+    PatPile* play[7];
+    PatPile* target[4];
 
     KlondikePile *pile;
-    Card::Rank target_tops[4];
+    KCardDeck::Rank target_tops[4];
 
-private slots:
-    void gameTypeChanged();
+    friend class KlondikeSolver;
 };
 
-class KlondikePile : public Pile
+class KlondikePile : public PatPile
 {
 public:
-    KlondikePile( int _index, int _draw, const QString & objectName = QString() );
+    KlondikePile( KCardScene * cardScene, int _index, int _draw, const QString & objectName = QString() );
     int draw() const { return m_draw; }
     void setDraws( int _draw );
     virtual void layoutCards( int duration );
